@@ -25,7 +25,6 @@
 #include "kcrash.h"
 
 #include <config-kcrash.h>
-#include <config-strlcpy.h>
 
 #include <string.h>
 #include <signal.h>
@@ -36,6 +35,7 @@
 #ifndef Q_OS_WIN
 #include <sys/resource.h>
 #include <sys/un.h>
+#include <errno.h>
 #else
 #include <qt_windows.h>
 #endif
@@ -47,7 +47,7 @@
 #include <kstartupinfo.h>
 
 #include <QDebug>
-#include <QCoreApplication>
+#include <QGuiApplication>
 #include <QtCore/QFileInfo>
 #include <QtCore/QDir>
 #include <QStandardPaths>
@@ -209,10 +209,16 @@ void KCrash::setDrKonqiEnabled(bool enabled)
 {
     s_launchDrKonqi = enabled ? 1 : 0;
     if (s_launchDrKonqi && !s_drkonqiPath) {
-        s_drkonqiPath = qstrdup(CMAKE_INSTALL_PREFIX "/" LIBEXEC_INSTALL_DIR "/drkonqi");
-        if (!QFile::exists(QLatin1String(s_drkonqiPath))) {
-            qWarning() << "Could not find drkonqi at" << s_drkonqiPath;
-            s_launchDrKonqi = 0;
+        const QList<QString> paths = QFile::decodeName(qgetenv("LIBEXEC_PATH")).split(QLatin1Char(':'));
+        const QString exec = QStandardPaths::findExecutable(QLatin1String("drkonqi"), paths);
+        if (exec.isEmpty()) {
+            s_drkonqiPath = qstrdup(CMAKE_INSTALL_PREFIX "/" LIBEXEC_INSTALL_DIR "/drkonqi");
+            if (!QFile::exists(QLatin1String(s_drkonqiPath))) {
+                qWarning() << "Could not find drkonqi at" << s_drkonqiPath;
+                s_launchDrKonqi = 0;
+            }
+        } else {
+            s_drkonqiPath = qstrdup(qPrintable(exec));
         }
     }
 
@@ -272,6 +278,7 @@ KCrash::setCrashHandler(HandlerType handler)
         // Prepare this now to avoid mallocs in the crash handler.
         char *display = getDisplay();
         const QString socketFileName = QString::fromLatin1("kdeinit5_%1").arg(QLatin1String(display));
+        free(display);
         QByteArray socketName = QFile::encodeName(QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) +
                                 QLatin1Char('/') + socketFileName);
         s_kdeinit_socket_file = qstrdup(socketName.constData());
@@ -404,19 +411,27 @@ KCrash::defaultCrashHandler(int sig)
             return;
         }
 
-        const char *argv[27];  // don't forget to update this
+        const char *argv[29];  // don't forget to update this
         int i = 0;
 
         // argument 0 has to be drkonqi
         argv[i++] = s_drkonqiPath;
 
+        const QByteArray platformName = QGuiApplication::platformName().toUtf8();
+        if (!platformName.isEmpty()) {
+            argv[i++] = "-platform";
+            argv[i++] = platformName.constData();
+        }
+
 #if HAVE_X11
-        // start up on the correct display
-        argv[i++] = "-display";
-        if (QX11Info::display()) {
-            argv[i++] = XDisplayString(QX11Info::display());
-        } else {
-            argv[i++] = getenv("DISPLAY");
+        if (platformName == QByteArrayLiteral("xcb")) {
+            // start up on the correct display
+            argv[i++] = "-display";
+            if (QX11Info::display()) {
+                argv[i++] = XDisplayString(QX11Info::display());
+            } else {
+                argv[i++] = getenv("DISPLAY");
+            }
         }
 #endif
 
@@ -462,11 +477,11 @@ KCrash::defaultCrashHandler(int sig)
             }
         }
 
-        char sidtxt[256];
-        if (!KStartupInfo::startupId().isNull()) {
+        // make sure the constData() pointer remains valid when we call startProcess by making a copy
+        QByteArray startupId = KStartupInfo::startupId();
+        if (!startupId.isNull()) {
             argv[i++] = "--startupid";
-            strlcpy(sidtxt, KStartupInfo::startupId().constData(), sizeof(sidtxt));
-            argv[i++] = sidtxt;
+            argv[i++] = startupId.constData();
         }
 
         if (s_flags & SaferDialog) {
@@ -676,6 +691,7 @@ static pid_t startDirectly(const char *argv[])
         fprintf(stderr, "KCrash failed to fork(), errno = %d\n", errno);
         return 0;
     case 0:
+        setgroups(0, 0); // Remove any extraneous groups
         if (setgid(getgid()) < 0 || setuid(getuid()) < 0) {
             _exit(253);    // This cannot happen. Theoretically.
         }
